@@ -32,10 +32,7 @@ def long_description():
   return long_description
 
 
-def version():
-  with codecs.open('VERSION.txt', 'r', 'utf-8') as f:
-    version = f.read().rstrip()
-    return version
+exec(open('src/sentencepiece/_version.py').read())
 
 
 def run_pkg_config(section, pkg_config_path=None):
@@ -60,22 +57,42 @@ def is_sentencepiece_installed():
     return False
 
 
+def get_cflags_and_libs(root):
+  cflags = ['-std=c++11', '-I' + os.path.join(root, 'include')]
+  libs = []
+  if os.path.exists(os.path.join(root, 'lib/pkgconfig/sentencepiece.pc')):
+    libs = [
+        os.path.join(root, 'lib/libsentencepiece.a'),
+        os.path.join(root, 'lib/libsentencepiece_train.a'),
+    ]
+  elif os.path.exists(os.path.join(root, 'lib64/pkgconfig/sentencepiece.pc')):
+    libs = [
+        os.path.join(root, 'lib64/libsentencepiece.a'),
+        os.path.join(root, 'lib64/libsentencepiece_train.a'),
+    ]
+  return cflags, libs
+
+
 class build_ext(_build_ext):
   """Override build_extension to run cmake."""
 
   def build_extension(self, ext):
-    pkg_config_path = None
-    if not is_sentencepiece_installed():
-      subprocess.check_call(['./build_bundled.sh', version()])
-      pkg_config_path = './bundled/lib/pkgconfig:./bundled/lib64/pkgconfig'
+    cflags, libs = get_cflags_and_libs('../build/root')
+    if len(libs) == 0:
+      cflags, libs = get_cflags_and_libs('./bundled/root')
+
+    if len(libs) == 0:
+      if is_sentencepiece_installed():
+        cflags = cflags + run_pkg_config('cflags')
+        libs = run_pkg_config('libs')
+      else:
+        subprocess.check_call(['./build_bundled.sh', __version__])
+        cflags, libs = get_cflags_and_libs('./bundled/root')
 
-    cflags = ['-std=c++11']
     # Fix compile on some versions of Mac OSX
     # See: https://github.com/neulab/xnmt/issues/199
     if sys.platform == 'darwin':
       cflags.append('-mmacosx-version-min=10.9')
-    cflags = cflags + run_pkg_config('cflags', pkg_config_path)
-    libs = run_pkg_config('libs', pkg_config_path)
     print('## cflags={}'.format(' '.join(cflags)))
     print('## libs={}'.format(' '.join(libs)))
     ext.extra_compile_args = cflags
@@ -84,11 +101,23 @@ class build_ext(_build_ext):
 
 
 if os.name == 'nt':
-  cflags = ['/MT', '/I..\\build\\root\\include']
-  libs = [
-      '..\\build\\root\\lib\\sentencepiece.lib',
-      '..\\build\\root\\lib\\sentencepiece_train.lib'
-  ]
+  # Must pre-install sentencepice into bundled directory.
+  arch = 'win32'
+  if sys.maxsize > 2**32:
+    arch = 'amd64'
+  if os.path.exists('..\\build\\root_{}\\lib'.format(arch)):
+    cflags = ['/MT', '/I..\\build\\root_{}\\include'.format(arch)]
+    libs = [
+        '..\\build\\root_{}\\lib\\sentencepiece.lib'.format(arch),
+        '..\\build\\root_{}\\lib\\sentencepiece_train.lib'.format(arch)
+    ]
+  else:
+    cflags = ['/MT', '/I..\\build\\root\\include']
+    libs = [
+        '..\\build\\root\\lib\\sentencepiece.lib',
+        '..\\build\\root\\lib\\sentencepiece_train.lib'
+    ]
+
   SENTENCEPIECE_EXT = Extension(
       'sentencepiece._sentencepiece',
       sources=['src/sentencepiece/sentencepiece_wrap.cxx'],
@@ -108,13 +137,14 @@ setup(
     description='SentencePiece python wrapper',
     long_description=long_description(),
     long_description_content_type='text/markdown',
-    version=version(),
+    version=__version__,
     package_dir={'': 'src'},
     url='https://github.com/google/sentencepiece',
     license='Apache',
     platforms='Unix',
     py_modules=[
-        'sentencepiece/__init__', 'sentencepiece/sentencepiece_model_pb2',
+        'sentencepiece/__init__', 'sentencepiece/_version',
+        'sentencepiece/sentencepiece_model_pb2',
         'sentencepiece/sentencepiece_pb2'
     ],
     ext_modules=[SENTENCEPIECE_EXT],
