@@ -33,8 +33,9 @@
 namespace sentencepiece {
 
 // "_this_is_a_pen" => ["_this", "_is", "_a", "_pen"]
-std::vector<absl::string_view> SplitIntoWords(absl::string_view text,
-                                              bool add_ws_as_suffix = false);
+std::vector<absl::string_view> SplitIntoWords(
+    absl::string_view text, bool treat_ws_as_suffix = false,
+    bool allow_ws_only_pieces = false);
 
 // Converts byte (0-255) to piece (e.g., 58 -> "<0x3A>").
 std::string ByteToPiece(unsigned char c);
@@ -106,12 +107,42 @@ class ModelInterface {
     return EncodeResult();
   }
 
+  // Sample `samples` many tokenisations from the segmentation lattice
+  // If `wor` is true, the samples are taken without replacement, and the scores
+  // are the inclusion probabilities of the elements in the sample; otherwise
+  // the samples are taken with replacement and the scores are the log-probs of
+  // sample elements
+  // If `include_best` is true, the best tokenisation is always included in the
+  // sample, and the remaining elements are sampled excluding the best.
+  virtual NBestEncodeResult SampleEncodeAndScore(absl::string_view normalized,
+                                                 float alpha, int samples,
+                                                 bool wor,
+                                                 bool include_best) const {
+    LOG(ERROR) << "Not implemented.";
+    return {{EncodeResult(), 0.0}};
+  }
+
+  // Calculates the entropy of the segmentation lattice with inverse temperature
+  // `theta`.
+  // Uses a novel dynamic program to calculate the entropy.
+  virtual float CalculateEntropy(absl::string_view normalized,
+                                 float theta) const {
+    LOG(ERROR) << "Not implemented.";
+    return 0.0;
+  }
+
   // Return true if SampleEncode returns a valid result.
   virtual bool IsSampleEncodeAvailable() const { return false; }
 
   // Return true if NBestEncode returns a valid result.
   virtual bool IsNBestEncodeAvailable() const { return false; }
 
+  // Return true if SampleEncodeAndScore returns a valid result.
+  virtual bool IsSampleEncodeAndScoreAvailable() const { return false; }
+
+  // Return true if CalculateEntropy returns a valid result.
+  virtual bool IsCalculateEntropyAvailable() const { return false; }
+
   // Returns the vocab id of `piece`.
   // Returns UNK(0) if `piece` is unknown
   virtual int PieceToId(absl::string_view piece) const;
@@ -124,7 +155,10 @@ class ModelInterface {
 
   // Returns the size of sentence pieces, which is the same
   // as the size of vocabulary for NMT.
-  virtual int GetPieceSize() const { return model_proto_->pieces_size(); }
+  virtual int GetPieceSize() const {
+    if (!model_proto_) return 0;
+    return model_proto_->pieces_size();
+  }
 
   // Returns the score of `id`.
   // Score represents a log probability of the piece.
