@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.!
 
+#include "sentencepiece_processor.h"
+
 #include <map>
 #include <set>
 #include <utility>
@@ -22,7 +24,6 @@
 #include "model_interface.h"
 #include "normalizer.h"
 #include "sentencepiece.pb.h"
-#include "sentencepiece_processor.h"
 #include "third_party/absl/memory/memory.h"
 #include "third_party/absl/strings/numbers.h"
 #include "third_party/absl/strings/str_cat.h"
@@ -333,6 +334,56 @@ util::Status SentencePieceProcessor::SampleEncode(absl::string_view input,
   return util::OkStatus();
 }
 
+util::Status SentencePieceProcessor::SampleEncodeAndScore(
+    absl::string_view input, int num_samples, float theta, bool wor,
+    bool include_best,
+    std::vector<std::pair<std::vector<std::string>, float>> *pieces) const {
+  CHECK_OR_RETURN_STATUS_STL(pieces);
+
+  NBestSentencePieceText spt;
+  RETURN_IF_ERROR(
+      SampleEncodeAndScore(input, num_samples, theta, wor, include_best, &spt));
+
+  pieces->clear();
+  pieces->reserve(spt.nbests_size());
+
+  for (const auto &nbest : spt.nbests()) {
+    std::vector<std::string> result;
+    result.reserve(nbest.pieces_size());
+    for (const auto &sp : nbest.pieces()) {
+      result.emplace_back(sp.piece());
+    }
+    pieces->emplace_back(result, nbest.score());
+  }
+
+  return util::OkStatus();
+}
+
+util::Status SentencePieceProcessor::SampleEncodeAndScore(
+    absl::string_view input, int num_samples, float theta, bool wor,
+    bool include_best,
+    std::vector<std::pair<std::vector<int>, float>> *ids) const {
+  CHECK_OR_RETURN_STATUS_STL(ids);
+
+  NBestSentencePieceText spt;
+  RETURN_IF_ERROR(
+      SampleEncodeAndScore(input, num_samples, theta, wor, include_best, &spt));
+
+  ids->clear();
+  ids->reserve(spt.nbests_size());
+
+  for (const auto &nbest : spt.nbests()) {
+    std::vector<int> result;
+    result.reserve(nbest.pieces_size());
+    for (const auto &sp : nbest.pieces()) {
+      result.emplace_back(sp.id());
+    }
+    ids->emplace_back(result, nbest.score());
+  }
+
+  return util::OkStatus();
+}
+
 util::Status SentencePieceProcessor::PopulateSentencePieceText(
     absl::string_view input, absl::string_view normalized,
     const std::vector<size_t> &norm_to_orig, const EncodeResult &result,
@@ -548,18 +599,24 @@ util::Status SentencePieceProcessor::Decode(
   if (model_proto_ && model_proto_->trainer_spec().has_unk_surface())
     unk_surface = model_proto_->trainer_spec().unk_surface().c_str();
 
-  auto DecodeSentencePiece = [&](absl::string_view piece, int id,
-                                 bool is_bos_ws) -> std::string {
-    if (IsControl(id)) {  // <s>, </s>
-      return "";          // invisible symbol.
+  // Returns decoded piece and a boolean indicating if the function has consumed
+  // a bos whitespace token (a piece starting with a kSpaceSymbol). This is used
+  // to strip only the first whitespace token from the decoded sequence for
+  // add_dummy_prefix.
+  auto DecodeSentencePiece =
+      [&](absl::string_view piece, int id,
+          bool is_bos_ws) -> std::pair<std::string, bool> {
+    if (IsControl(id)) {                 // <s>, </s>
+      return std::make_pair("", false);  // invisible symbol.
     } else if (IsUnknown(id)) {
       if (IdToPiece(id) == piece) {  // <unk>
-        return unk_surface;
+        return std::make_pair(unk_surface, false);
       } else {  // return piece when piece is not <unk>.
-        return std::string(piece);
+        return std::make_pair(std::string(piece), false);
       }
     }
 
+    bool has_bos_ws = false;  // whether the token starts with a kSpaceSymbol
     if (is_bos_ws &&
         (!model_proto_ ||
          (model_proto_ &&
@@ -567,10 +624,17 @@ util::Status SentencePieceProcessor::Decode(
            model_proto_->normalizer_spec().remove_extra_whitespaces())))) {
       // Consume if the current position is bos and
       // piece starts with kSpaceSymbol.
-      absl::ConsumePrefix(&piece, kSpaceSymbol);
+      has_bos_ws = absl::ConsumePrefix(&piece, kSpaceSymbol);
+
+      if (model_proto_ &&
+          model_proto_->normalizer_spec().remove_extra_whitespaces()) {
+        // if we are removing extra whitespace, we remove all leading whitespace
+        has_bos_ws = false;
+      }
     }
 
-    return absl::StrReplaceAll(piece, {{kSpaceSymbol, " "}});
+    return std::make_pair(absl::StrReplaceAll(piece, {{kSpaceSymbol, " "}}),
+                          has_bos_ws);
   };
 
   for (const std::string &w : pieces) {
@@ -644,12 +708,23 @@ util::Status SentencePieceProcessor::Decode(
   };
 
   int byte_start = 0;
+  bool is_bos_ws = true;  // whether we expect a bos ws token to consume.
+  bool bos_ws_seen = false;
+  std::string decoded;
+
   for (int i = 0; i < spt->pieces_size(); ++i) {
     const auto &sp = spt->pieces(i);
     if (!IsByte(sp.id())) {
       RETURN_IF_ERROR(ProcessBytePieces(byte_start, i));
+
+      // if we have seen a bos_ws token or any non-empty token
+      if (bos_ws_seen || !text->empty()) is_bos_ws = false;
+
       byte_start = i + 1;
-      SetSurface(i, DecodeSentencePiece(sp.piece(), sp.id(), text->empty()));
+      std::tie(decoded, bos_ws_seen) =
+          DecodeSentencePiece(sp.piece(), sp.id(), is_bos_ws);
+
+      SetSurface(i, decoded);
     }
   }
   RETURN_IF_ERROR(ProcessBytePieces(byte_start, spt->pieces_size()));
@@ -809,6 +884,15 @@ util::Status SentencePieceProcessor::ApplyExtraOptions(
         piece->set_piece(model_->bos_piece().data(),
                          model_->bos_piece().size());
       } break;
+      case UNK_PIECE: {
+        for (int i = 0; i < spt->pieces_size(); ++i) {
+          auto *piece = spt->mutable_pieces(i);
+          if (IsUnknown(piece->id())) {
+            piece->set_piece(model_->unk_piece().data(),
+                             model_->unk_piece().size());
+          }
+        }
+      } break;
       default:
         return util::InternalError("unknown extra_option type.");
     }
@@ -831,7 +915,9 @@ util::Status SentencePieceProcessor::ParseExtraOptions(
   static std::map<absl::string_view, SentencePieceProcessor::ExtraOption>
       extra_option_map = {{"bos", SentencePieceProcessor::BOS},
                           {"eos", SentencePieceProcessor::EOS},
-                          {"reverse", SentencePieceProcessor::REVERSE}};
+                          {"reverse", SentencePieceProcessor::REVERSE},
+                          {"unk", SentencePieceProcessor::UNK_PIECE},
+                          {"unk_piece", SentencePieceProcessor::UNK_PIECE}};
   for (const auto &s : absl::StrSplit(extra_option, ":")) {
     const auto it = extra_option_map.find(s);
     CHECK_OR_RETURN(it != extra_option_map.end())
