@@ -163,6 +163,7 @@ namespace normalizer {
 class Normalizer;
 }  // namespace normalizer
 
+#ifndef SWIG
 // Defines the multiple versions of encoder within each model. Currently only
 // the Unigram model has an optimized encoder.
 enum class EncoderVersion {
@@ -170,13 +171,16 @@ enum class EncoderVersion {
   kOriginal    // The original encoder (user may choose to fall back to this
                // just in case).
 };
+#endif
 
+#ifndef SWIGGO
 namespace util {
 // Redefine std::string for serialized_proto interface as Python's string is
 // a Unicode string. We can enforce the return value to be raw byte sequence
 // with SWIG's typemap.
 using bytes = std::string;
 }  // namespace util
+#endif
 
 class SentencePieceProcessor {
  public:
@@ -250,6 +254,7 @@ class SentencePieceProcessor {
   virtual util::Status Decode(const std::vector<int> &ids,
                               std::string *detokenized) const;
 
+#ifndef SWIG
   // Sets the encoder version. Normally users do not need to call this function.
   // But they can call this fucntion just in case if they want to fall back to
   // the original encoder.
@@ -257,6 +262,7 @@ class SentencePieceProcessor {
 
   // Returns the current encoder version in use.
   virtual EncoderVersion GetEncoderVersion() const;
+#endif
 
   //////////////////////////////////////////////////////////////
   // NBest API.
@@ -297,6 +303,20 @@ class SentencePieceProcessor {
   virtual util::Status SampleEncode(absl::string_view input, int nbest_size,
                                     float alpha, std::vector<int> *ids) const;
 
+  //////////////////////////////////////////////////////////////
+  // SampleEncodeAndScore API.
+  // Similar to SampleEncode, but returns samples results.
+  virtual util::Status SampleEncodeAndScore(
+      absl::string_view input, int num_samples, float theta, bool wor,
+      bool include_best,
+      std::vector<std::pair<std::vector<std::string>, float>> *pieces) const;
+
+  // Same as above, but returns a sequence of ids.
+  virtual util::Status SampleEncodeAndScore(
+      absl::string_view input, int num_samples, float theta, bool wor,
+      bool include_best,
+      std::vector<std::pair<std::vector<int>, float>> *ids) const;
+
   //////////////////////////////////////////////////////////////
   // Advanced API returning SentencePieceText, which manages
   // utf8-byte alignments between user-input/detokenized text
@@ -315,6 +335,17 @@ class SentencePieceProcessor {
   virtual util::Status SampleEncode(absl::string_view input, int nbest_size,
                                     float alpha, SentencePieceText *spt) const;
 
+  // Samples N segmentation and returns the scores as well
+  virtual util::Status SampleEncodeAndScore(
+      absl::string_view input, int samples, float theta, bool wor,
+      bool include_best, NBestSentencePieceText *samples_spt) const;
+
+#ifndef SWIG
+  // Calculate entropy of possible tokenisations
+  virtual util::Status CalculateEntropy(absl::string_view input, float theta,
+                                        float *entropy) const;
+#endif
+
   // Given a sequence of pieces, decodes it into SentencePieceText.
   virtual util::Status Decode(const std::vector<std::string> &pieces,
                               SentencePieceText *spt) const;
@@ -374,6 +405,22 @@ class SentencePieceProcessor {
                                 nbest_size, alpha);
   }
 
+  virtual std::vector<std::pair<std::vector<std::string>, float>>
+  SampleEncodeAndScoreAsPieces(absl::string_view input, int num_samples,
+                               float theta, bool wor, bool include_best) const {
+    using _T = std::vector<std::pair<std::vector<std::string>, float>>;
+    DEFINE_SPP_DIRECT_FUNC_IMPL(SampleEncodeAndScore, _T, input, num_samples,
+                                theta, wor, include_best);
+  }
+
+  virtual std::vector<std::pair<std::vector<int>, float>>
+  SampleEncodeAndScoreAsIds(absl::string_view input, int num_samples,
+                            float theta, bool wor, bool include_best) const {
+    using _T = std::vector<std::pair<std::vector<int>, float>>;
+    DEFINE_SPP_DIRECT_FUNC_IMPL(SampleEncodeAndScore, _T, input, num_samples,
+                                theta, wor, include_best);
+  }
+
   virtual std::string DecodePieces(
       const std::vector<std::string> &pieces) const {
     DEFINE_SPP_DIRECT_FUNC_IMPL(Decode, std::string, pieces);
@@ -383,6 +430,10 @@ class SentencePieceProcessor {
     DEFINE_SPP_DIRECT_FUNC_IMPL(Decode, std::string, ids);
   }
 
+  virtual float CalculateEntropy(absl::string_view text, float theta) const {
+    DEFINE_SPP_DIRECT_FUNC_IMPL(CalculateEntropy, float, text, theta);
+  }
+
 #undef DEFINE_SPP_DIRECT_FUNC_IMPL
 
   // They are used in Python interface. Returns serialized proto.
@@ -468,7 +519,7 @@ class SentencePieceProcessor {
   util::bytes serialized_model_proto() const;
 
  private:
-  enum ExtraOption { REVERSE, BOS, EOS };
+  enum ExtraOption { REVERSE, BOS, EOS, UNK_PIECE };
 
   util::Status ParseExtraOptions(absl::string_view extra_option,
                                  std::vector<ExtraOption> *extra_options) const;
