@@ -14,6 +14,7 @@
 
 #include "trainer_interface.h"
 
+#include <algorithm>
 #include <cstdlib>
 #include <memory>
 #include <set>
@@ -29,6 +30,8 @@
 #include "sentencepiece_trainer.h"
 #include "third_party/absl/container/flat_hash_map.h"
 #include "third_party/absl/memory/memory.h"
+#include "third_party/absl/random/distributions.h"
+#include "third_party/absl/random/random.h"
 #include "third_party/absl/strings/numbers.h"
 #include "third_party/absl/strings/str_cat.h"
 #include "third_party/absl/strings/str_format.h"
@@ -86,6 +89,10 @@ util::Status VerifySpec(const TrainerSpec &trainer_spec) {
   return util::OkStatus();
 }
 
+bool is_unicode_decimal_number(char32 c) {
+  return (c >= 0x30 && c <= 0x39) || (c >= 0xff10 && c <= 0xff19);
+}
+
 class SentenceSelector {
  public:
   using Sampler = random::ReservoirSampler<TrainerInterface::Sentence>;
@@ -210,9 +217,10 @@ bool TrainerInterface::IsValidSentencePiece(
   constexpr unicode_script::ScriptType kAnyType =
       static_cast<unicode_script::ScriptType>(-1);
 
-  auto is_number = [](char32 c) { return (c >= 0x30 && c <= 0x39); };
-
   unicode_script::ScriptType prev_script = kAnyType;
+  bool all_whitespace_piece =
+      std::all_of(sentencepiece.begin(), sentencepiece.end(),
+                  [](char32 c) { return c == kWSChar; });
 
   for (size_t pos = 0; pos < sentencepiece.size(); ++pos) {
     const char32 c = sentencepiece[pos];
@@ -235,25 +243,30 @@ bool TrainerInterface::IsValidSentencePiece(
     }
 
     if (c == kWSChar) {
-      // Only allows whitespace to appear as a prefix of piece.
+      // Only allows whitespace to appear as a prefix of piece unless
+      // allow_whitespace_only_pieces is True.
       // When split_by_whitespace is false, we allow whitespaces to
       // appear in the middle, "foo_bar", but do not allow them
       // to appear as suffix, "foo_bar_".
       // Regardless of the setting of split_by_whitespace,
       // whitespace is treated as a prefix/infix of symbol or
-      // independent symbol.
-      if (trainer_spec_.treat_whitespace_as_suffix()) {
-        if ((trainer_spec_.split_by_whitespace() &&
-             pos < sentencepiece.size() - 1) ||
-            (!trainer_spec_.split_by_whitespace() &&
-             pos < sentencepiece.size() - 1 && pos == 0)) {
-          return false;
-        }
-      } else {
-        if ((trainer_spec_.split_by_whitespace() && pos > 0) ||
-            (!trainer_spec_.split_by_whitespace() && pos > 0 &&
-             pos == sentencepiece.size() - 1)) {
-          return false;
+      // independent symbol, unless allow_whitespace_only_pieces() is true,
+      // in which case whitespace only pieces can occur.
+      if (!trainer_spec_.allow_whitespace_only_pieces() ||
+          !all_whitespace_piece) {
+        if (trainer_spec_.treat_whitespace_as_suffix()) {
+          if ((trainer_spec_.split_by_whitespace() &&
+               pos < sentencepiece.size() - 1) ||
+              (!trainer_spec_.split_by_whitespace() &&
+               pos < sentencepiece.size() - 1 && pos == 0)) {
+            return false;
+          }
+        } else {
+          if ((trainer_spec_.split_by_whitespace() && pos > 0) ||
+              (!trainer_spec_.split_by_whitespace() && pos > 0 &&
+               pos == sentencepiece.size() - 1)) {
+            return false;
+          }
         }
       }
     } else {
@@ -263,13 +276,15 @@ bool TrainerInterface::IsValidSentencePiece(
       if (s == unicode_script::U_Hiragana || s == unicode_script::U_Katakana ||
           c == 0x30FC) {  // long vowel sound (Katakana) should be Katakana
         s = unicode_script::U_Han;
+      } else if (s == unicode_script::U_Inherited) {
+        s = prev_script;
       }
 
-      if (!trainer_spec_.split_by_number() && is_number(c)) {
+      if (!trainer_spec_.split_by_number() && is_unicode_decimal_number(c)) {
         s = kAnyType;
       }
 
-      if (trainer_spec_.split_digits() && is_number(c)) {
+      if (trainer_spec_.split_digits() && is_unicode_decimal_number(c)) {
         if (sentencepiece.size() > 1) return false;
       }
 
@@ -286,6 +301,22 @@ bool TrainerInterface::IsValidSentencePiece(
   return true;
 }
 
+template <typename T>
+void AddDPNoise(const TrainerSpec &trainer_spec, absl::SharedBitGen &generator,
+                T *to_update) {
+  if (trainer_spec.differential_privacy_noise_level() > 0) {
+    float random_num = absl::Gaussian<float>(
+        generator, 0, trainer_spec.differential_privacy_noise_level());
+
+    *to_update =
+        std::round(std::max(0.f, random_num + static_cast<float>(*to_update)));
+  }
+  // Clip anything below the clipping threshold to 0.
+  if (*to_update < trainer_spec.differential_privacy_clipping_threshold()) {
+    *to_update = 0;
+  }
+}
+
 util::Status TrainerInterface::LoadSentences() {
   RETURN_IF_ERROR(status());
   CHECK_OR_RETURN(sentences_.empty());
@@ -377,6 +408,7 @@ END:
     LOG(INFO) << "Sampled " << sentences_.size() << " sentences from "
               << selector.total_size() << " sentences.";
   }
+
   if (too_long_lines > 0)
     LOG(INFO) << "Skipped " << too_long_lines << " too long sentences.";
   if (self_test_samples_.size() > 0)
@@ -420,6 +452,54 @@ END:
     }
   }
 
+  // If DP is required, add the noise/clip the input.
+  if (trainer_spec_.enable_differential_privacy()) {
+    if (trainer_spec_.input_format() != "tsv") {
+      LOG(ERROR)
+          << "Dp version will not work correctly with text input format.";
+    }
+    if (trainer_spec_.differential_privacy_noise_level() <= 0) {
+      LOG(WARNING) << "Private version with <=0 noise level will give "
+                      "infinity epsilon guarantees.";
+    }
+    if (trainer_spec_.differential_privacy_clipping_threshold() <= 0) {
+      LOG(WARNING) << "Private version with <=0 clipping threshold will give "
+                      "infinity epsilon guarantees.";
+    }
+
+    // Add noise to all the sentences via threadpool.
+
+    // This line is mainly for tests with small num of sentences.
+    const auto num_workers =
+        std::min<uint64>(trainer_spec_.num_threads(), sentences_.size() - 1);
+
+    {
+      auto pool = absl::make_unique<ThreadPool>(num_workers);
+      pool->StartWorkers();
+      for (int n = 0; n < num_workers; ++n) {
+        pool->Schedule([&, n]() {
+          // One per thread generator.
+          absl::SharedBitGen generator;
+          for (size_t i = n; i < sentences_.size(); i += num_workers) {
+            AddDPNoise<int64>(trainer_spec_, generator,
+                              &(sentences_[i].second));
+          }
+        });
+      }
+    }
+
+    // Remove zero freq elements.
+    const auto before_size = sentences_.size();
+    auto it = std::remove_if(sentences_.begin(), sentences_.end(),
+                             [](const Sentence &s) { return s.second <= 0; });
+    const auto new_size = std::distance(sentences_.begin(), it);
+    const int num_erased = before_size - new_size;
+    sentences_.erase(it, sentences_.end());
+
+    LOG(INFO) << "DP noise resulted in " << 1.0 * num_erased / before_size
+              << " fraction of sentences removed.";
+  }
+
   // Count character frequencies.
   int64 all_chars_count = 0;
   // A map from a character to {is_required_char, character count}.
@@ -518,7 +598,8 @@ void TrainerInterface::SplitSentencesByWhitespace() {
   absl::flat_hash_map<std::string, int64> tokens;
   for (const auto &s : sentences_) {
     for (const auto &w :
-         SplitIntoWords(s.first, trainer_spec_.treat_whitespace_as_suffix())) {
+         SplitIntoWords(s.first, trainer_spec_.treat_whitespace_as_suffix(),
+                        trainer_spec_.allow_whitespace_only_pieces())) {
       tokens[std::string(w)] += s.second;
     }
   }
@@ -603,6 +684,7 @@ util::Status TrainerInterface::Serialize(ModelProto *model_proto) const {
 util::Status TrainerInterface::SaveModel(absl::string_view filename) const {
   LOG(INFO) << "Saving model: " << filename;
   ModelProto model_proto;
+
   RETURN_IF_ERROR(Serialize(&model_proto));
 
   auto output = filesystem::NewWritableFile(filename.data(), true);
@@ -618,6 +700,14 @@ util::Status TrainerInterface::SaveVocab(absl::string_view filename) const {
   auto output = filesystem::NewWritableFile(filename);
   RETURN_IF_ERROR(output->status());
 
+  for (const auto &piece : model_proto.pieces()) {
+    if (piece.piece().find_first_of(" \t\r\n") != std::string::npos) {
+      LOG(WARNING) << "The piece [" << piece.piece()
+                   << "] contains escaped characters that break the format of "
+                   << filename;
+    }
+  }
+
   if (trainer_spec_.vocabulary_output_piece_score()) {
     for (const auto &piece : model_proto.pieces()) {
       std::ostringstream os;
